#!/usr/bin/perl

# Multihost crontab support

use strict;
use Data::Dumper;
use Getopt::Std;
use File::Copy;
use Sys::Hostname;

use FindBin;
use lib "$FindBin::Bin/lib";
use lib "$FindBin::Bin/etc";

# amherst provided
use common;
use config;

$cfg{'verbose'} = 1;

#
# do not change these, else you have double the cron entries
#
my $cron_header = "#--- MuLtIhOsT CrOn BeGiN";
my $cron_footer = "#--- MuLtIhOsT CrOn EnD";

my $host = hostname;
logger("cron> updating $host");

if (!defined($cfg{'cron'}{$host}) )
{
	logger_err("cron_err> unable to find a definition for this host=$host");
	exit(1);
}
if ( !cron_read($host) )
{
	logger_err("cron_err> no changes made, error encountered");
	exit(1);
}
my $cronfile = cron_edit($host);
logger("cron> updating cron with $cronfile");
if ( !cron_update($cronfile) )
{
	logger_err("cron_err> updating running crontab failed");
	exit(1);
}
exit(0);

#-------------------------------------------------------------------------------
# read current crontab file
#-------------------------------------------------------------------------------
sub cron_read
{
	my ($host) = @_;
	
	my $tmp = "$cfg{'state_directory'}/cron.$host.current";
	logger("cron> reading current crontab settings into $tmp");
	
	my $cmd = "crontab -l > $tmp";
	logger("cron> $cmd");
	system($cmd);
	if ( $? != 0 )
	{
		logger_err("cron_err> cmd $cmd");
		logger_err("cron_err> child exit value ".($?>>8));
		return undef;
	}
	
	return 1;	
}

#-------------------------------------------------------------------------------
# edit current crontab file (parse for header and footer and replace the section)
#-------------------------------------------------------------------------------
sub cron_edit
{
	my ($host) = @_;
	
	my $infile = "$cfg{'state_directory'}/cron.$host.current";
	my $outfile = "$cfg{'state_directory'}/cron.$host.new";
	
	open(FIN, "<$infile") or die "Could not open $infile: $!";
	open(FOUT, ">$outfile") or die "Could not open $outfile: $!";

	#
	# as we find the begin header, ignore lines, until the footer
	# at the end append the replacement cron configuration
	#
	my @lines;
	my $ignore_lines = 0;
	while(<FIN>)
	{
		my $line = $_;
		chomp($line);
		
		if ( $line =~ /^$cron_header:/ )
		{
			$ignore_lines = 1;
		}
		
		if ( !$ignore_lines )
		{
			push @lines, $line;
		}
		
		if ( $line =~ /^$cron_footer:/ )
		{
			$ignore_lines = 0;
		}
	}	
	
	#
	# remove blank lines at the end of the array (otherwise we end up
	# adding more blank lines in between the last real line and our
	# header
	#
	while(1)
	{
		if ( $lines[scalar(@lines)-1] ne "" )
		{
			last;
		}
		pop @lines;
	}
	push @lines, "";
		
	push @lines, "$cron_header: $host (".date_time_string().")";
	push @lines, $cfg{'cron'}{$host};
	push @lines, "$cron_footer: $host";
	
	#print Dumper \@lines;

	my $out = join("\n", @lines);
	print FOUT "$out\n";

	close(FIN);
	close(FOUT);
	
	return $outfile;
}

#-------------------------------------------------------------------------------
# update running crontab
#-------------------------------------------------------------------------------
sub cron_update
{
	my ($cronfile) = @_;

	my $cmd = "crontab < $cronfile";
	logger("cron> $cmd");
	system($cmd);
	if ( $? != 0 )
	{
		logger_err("cron_err> cmd $cmd");
		logger_err("cron_err> child exit value ".($?>>8));
		return undef;
	}
	
	return 1;	
}

