#!/usr/bin/bash

#
# ddless maintenance script
#
# this script generates the current ddless checksums of volumes and compares
# them to the currently defined ddless checksum. If there is a difference
# the defined ddless checksum is replaced.
#
# this script differs from snapcheck in that it does not operate on
# snapshots but rather the actual volume.
#
ROOT=$(dirname $0)

zpools=$(zpool list -H -o name)

function process_ddless()
{
	local zpool="$1"
	local ddless_file="$2"
	local zvol=$(echo "$ddless_file" | basename "$ddless_file" .ddless)
	local device="/dev/zvol/rdsk/$zpool/${zvol}_bk"
	
	echo "maint> ddless_file: $ddless_file"
	echo "maint> device: $device"
	if [ ! -e "$device" ]; then
		echo "maint_err> device $device not found"
		return
	fi
	
	md5_repo=$(md5sum "$ddless_file" | cut -f1 -d' ')
	echo "md5_repo: $md5_repo "

	ddless_new="/tmp/new_$$.ddless";
	$ROOT/bin/ddless -s "$device" -c "$ddless_new" -w 8
	if [ ! -e "$ddless_new" ]; then
		echo "file ddless_new: $ddless_new not found"
		exit 1
	fi

	md5_new=$(md5sum "$ddless_new" | cut -f1 -d' ')
	echo " md5_new: $md5_new"

	if [ "$md5_repo" = "$md5_new" ]; then
		echo " OK"
		return
	fi
	echo " FAIL"
	
	mv -v "$ddless_file" "$ddless_file.bad"
	mv -v "$ddless_new" "$ddless_file"
}

function enum_ddless()
{
	local zpool="$1"
	local pattern="$2"
	
	#
	# check for ddless directory
	#
	if [ ! -d "/$zpool/ddless" ]; then
		echo "maint_err> no ddless directory found, skipping"
		return
	fi
	
	#
	# find files that are recent (7 days)
	#
	ddless_files=$(find /$zpool/ddless -maxdepth 1 -mtime -7 -name "$pattern.ddless")
	if [ -z "$ddless_files" ]; then
		echo "maint_err> no ddless files found, skipping"
		return
	fi
	for ddless_file in $ddless_files; do
		process_ddless "$zpool" "$ddless_file"
		echo
	done
}

function process_all_pools()
{
	local pattern="$1"

	for zpool in $zpools; do
		echo "maint> zpool: $zpool"
		enum_ddless "$zpool" "$pattern"
	done
}

pattern="$1"
if [ -z "$pattern" ]; then
	echo "specify ddless file name pattern, like lv-exc..."
	exit 1
fi
process_all_pools "$pattern"

