#
# SanTools Multi Host Configuration File
#
package config;
use strict;
require Exporter;

use File::Basename;
use Data::Dumper;

our @ISA = qw(Exporter);
our @EXPORT = qw(
	%cfg
	%cfg_db
	);

our %cfg = (

	#---------------------------------------------------------------------------
	# multihost cron
	#---------------------------------------------------------------------------
	'cron' =>
		{
			#
			# zfs-archive1
			#
			'zfs-archive1' => '
# hourly zfs tree refresh for tkzfs
3 * * * * /root/santools/tkzfs/server/zfs_tree.sh refresh > /dev/null

# 7:00am daily
3 7 * * *  /root/santools/snapcheck_job.sh

# 8:00am daily
3 8 * * *  /root/santools/devfsadm_job.sh > /dev/null 2>&1

# every 3 months on the 1st at 9:30 am
30 9 1 * * echo "reboot due to devfsadm kernel state required"

# 10:00pm daily
3 22 * * * /root/santools/snaprotate_job.sh

# 11:00pm daily
3 23 * * * /root/santools/eql_luncopy_job.sh MAIL_A

# 1:00pm daily
3 13 * * * /root/santools/eql_luncopy_job.sh FILE_A

# every 12 hours
3 15   * * * /root/santools/iet_luncopy_job.sh -a
3 3    * * * /root/santools/iet_luncopy_job.sh -a -i

			',

			#
			# zfs-archive2
			#
			'zfs-archive2' => '
# hourly zfs tree refresh for tkzfs
6 * * * * /root/santools/tkzfs/server/zfs_tree.sh refresh > /dev/null

# 7:00am daily
3 7 * * *  /root/santools/snapcheck_job.sh

# 8:00am daily
3 8 * * *  /root/santools/devfsadm_job.sh > /dev/null 2>&1

# every 3 months on the 1st at 9:30 am
30 9 1 * * echo "reboot due to devfsadm kernel state required"

# 10:00pm daily
6 22 * * * /root/santools/snaprotate_job.sh

# 11:00am daily
6 23 * * * /root/santools/eql_luncopy_job.sh MAIL_B

# 1:00pm daily
6 13 * * * /root/santools/eql_luncopy_job.sh FILE_A

# every 12 hours
3 15   * * * /root/santools/iet_luncopy_job.sh -a
3 3    * * * /root/santools/iet_luncopy_job.sh -a -i
			',
		},
	
	#---------------------------------------------------------------------------
	# directory locations
	#---------------------------------------------------------------------------
	'etc_directory' => dirname($0).'/etc',
	'log_directory' => dirname($0).'/log',
	'state_directory' => dirname($0).'/state',

	#---------------------------------------------------------------------------
	# tool locations
	#---------------------------------------------------------------------------
	'ddless' => dirname($0).'/bin/ddless',
	'ddless_blocksize_bytes' => 16384,		# use ddless -p to see this value
	'dd' => '/usr/gnu/bin/dd',

	#---------------------------------------------------------------------------
	# iet group configuration(s)
	#---------------------------------------------------------------------------
	'iet_cluster' =>
	{
		'iscsi-target-node1' =>
		{
			'discovery_ip' => 'x.x.x.x',
			'enabled' => 1,
			'zfs_prefix' => 'iscsi',
		},
		'iscsi-target-node2' =>
		{
			'discovery_ip' => 'x.x.x.x.',
			'enabled' => 1,
			'zfs_prefix' => 'iscsi',
		}
	},

	'iet_luncopy_map' =>
	{
		'zfs-archive1' =>
		{
			'pool_map' =>
			{
				'vg_satabeast8_vol0' => 'vg_satabeast7_vol0',
				'vg_satabeast8_vol1' => 'vg_satabeast7_vol1',
			},

			'zfs_options' =>
				'-o compression=on '.
				'-o refreservation=none '.
				'-o edu.amherst:snaprotate_schedule=SNAP_ROTATE_A',
		},
	 },
	
	#---------------------------------------------------------------------------
	# equallogic group configuration(s)
	#---------------------------------------------------------------------------
	'default_eqlogic' => 'g-elogic0',
	'g-elogic0' =>
		{
			'ip' => 'x.x.x.x',
			'user' => 'grpadmin',
			'pass' => 'grpadmin',
			
			'concurrent_workers' => 4,
		},

	#---------------------------------------------------------------------------
	# eqlogic luncopy map: there are multiple zfs archive hosts, each
	# responsible for specific ddless filter tags, writing data
	# to a specific zpool. Each zpool is required to have a file system called
	# zpool/ddless to store the ddless checksum files
	#---------------------------------------------------------------------------
	'eql_luncopy_map' =>
	{
		'zfs-archive1' =>
		{
			'expose_snapshots_to_ip' => 'x.x.x.x',

			'filter_pool_map' =>
			{
				'MAIL_A'		=> 'vg_satabeast1_vol0',
				# continue with filters...
			},

			'pool_default' => 'vg_satabeast1_vol1',
			'pool_zfs_options' =>
			{
				# production zfs-archive1
				'vg_satabeast1_vol0' => 
					'-o compression=on '.
					'-o refreservation=none '.
					'-o edu.amherst:snaprotate_schedule=SNAP_ROTATE_A',

			},
		},

		'zfs-archive2' =>
		{
			'expose_snapshots_to_ip' => 'x.x.x.x',
			'filter_pool_map' =>
			{
				'FILE_A'	      => 'vg_satabeast2_vol0',
				# continue with filters...
			},

			'pool_default' => 'vg_satabeast2_vol0',
			'pool_zfs_options' =>
			{
				# production zfs-archive2
				'vg_satabeast2_vol0' => 
					'-o compression=on '.
					'-o refreservation=none '.
					'-o edu.amherst:snaprotate_schedule=SNAP_ROTATE_A',
			},
		},
	},

	#---------------------------------------------------------------------------
	# snap rotate does not create snapshots, rather it rotates existing
	# snapshots. Snaprotate schedules are time based. A volume or zfs
	# object can only have a single schedule assigned.
	#---------------------------------------------------------------------------
	'snaprotate_schedules' =>
	{
		#
		# production schedule
		#
		'SNAP_ROTATE_A' =>
		[
			{
				'name' => '2MONTHS_DAILY_SHORT_TERM',
				'freq' => 'daily',
				'duration' => 60,	# implied unit is days, due to frequency
			},
			{
				'name' => '4MONTHS_WEEKLY_MEDIUM_TERM',
				'freq' => 'weekly',
				'duration' => 16,	# implied unit is weekly, due to frequency
			},
			{
				'name' => '6MONTHS_MONTHLY_LONG_TERM',
				'freq' => 'monthly',
				'duration' => 6,	# implied unit is monthly, due to frequency
			},
		],
		
	},

	#---------------------------------------------------------------------------
	# snapshot checker (verifies ddless checksum against existing snaphots)
	#---------------------------------------------------------------------------
	'snapcheck' =>
	{
		'check_count_per_run' => 4,
		'ddless_workers' => 8,
	},
	
	#---------------------------------------------------------------------------
	# zfs copy allows us to make certain archive copies of zvols - legal cases
	# the zfs copy code was initially intended for zfs replication for comstar.
	#---------------------------------------------------------------------------
	'zfscopy' =>
	{
		'init_channel' => '~/santools/ndd.sh',
		'socat' => '~/santools/bin/socat',
		'socat_buf' => 262144,
		'socat_rcv' => 'rcvbuf=2097152',
		'socat_snd' => 'nodelay,sndbuf=2097152',
		'port_inc' => '~/santools/port_inc_get.plx',
	},

);

#print Dumper \%cfg;
1;
