#
# SUN Disk Info (Steffen Plotner)
#
package SunDiskInfo;

use strict;
use POSIX;
require Exporter;

use common;
use Data::Dumper;

our @ISA = qw(Exporter);
our @EXPORT = qw(
	sun_disk_info
	sun_disk_size
        );

#-------------------------------------------------------------------------------
# Collect SUN disk information using prtconf -v
# Only single value items are detected.
# Only specific named attributes are parsed for.
# Some values are conversted from hex to decimal.
# This does not work for zfs zvols, use sun_disk_size instead
#-------------------------------------------------------------------------------
sub sun_disk_info
{
	my($device) = @_;

	my @fields = (
		'inquiry-serial-no',
		'target-port',
		'device-nblocks',
		'device-blksize',
		'client-guid',
		'devid',
		'inquiry-revision-id',
		'inquiry-product-id',
		'inquiry-vendor-id',
		'inquiry-device-type',
		'lun',
		);
		
	my %device_info;
	my $cmd = "prtconf -v $device 2> /dev/null";
	my $out = `$cmd`;
	my $field;
	foreach my $line (split /\n/, $out)
	{
		#
		# once we reach minor node info, bail out
		#
		if ( $line =~ /Device Minor Nodes/i )
		{
			last;
		}
		
		#
		# dynamically iterate over the possible fields
		# and find a match
		#
		#logger("line=$line");
		foreach my $fld (@fields)
		{
			if ( $line =~ /name='$fld'/ )
			{
				$field = $fld;
			}
		}
		#logger("field=$field");
		
		#
		# if previous iteration revealed a field
		#
		if ( defined($field) )
		{
			#
			# and there is a an associated value
			#
			if ($line =~ /value=(.*)/ )
			{
				#
				# capture and convert if needed
				#
				my $val = $1;
				$val =~ s/'//g;
				$device_info{$field} = $val;
				
				if ( $field eq 'device-nblocks' ||
					$field eq 'device-blksize' ||
					$field eq 'lun')
				{
					$device_info{$field} = hex($val);
				}
				$field = undef;
			}
		}
	}
	
	#
	# calc device size given blocks and block size
	#
	if ( !defined($device_info{'device-blksize'}) )
	{
		#
		# solaris 10 does not expose this, later versions do
		#
		$device_info{'device-blksize'} = 512;
	}

	$device_info{'disk_size_gb'} =
		($device_info{'device-nblocks'} * $device_info{'device-blksize'}) / 
				1024 / 1024 / 1024;

	$device_info{'disk_size_bytes'} =
		($device_info{'device-nblocks'} * $device_info{'device-blksize'});

	return %device_info;
}

#-------------------------------------------------------------------------------
# Open a given device, go to the end and return the seek'd position in bytes
#-------------------------------------------------------------------------------
sub sun_disk_size
{
	my($device) = @_;

	open(FH,$device);
	seek FH,0, SEEK_END;
	my $len_bytes = tell(FH);
	close(FH);
	
	return ($len_bytes);
}
