#!/usr/bin/perl

use strict;

# The /etc/path_to_inst file has to be
# cleaned up. Remove all entries that match

# "/scsi_vhci/disk@g.............................." ## "sd"

# but keep:

# "/scsi_vhci" 0 "scsi_vhci"

# Once that is done, unmount the zpools and reboot.

# This has to be done because each time solaris sees a new
# disk a minor device number is persistently assigned.

# There is no interface with the kernel that allows
# us to either control this or remove the entries.

# We could write code to remove the entries, however, the
# moment we write the changes, the kernel could have made
# a decision about a minor device number that we now would
# have overwritten - since there is no usable locking here,
# there is no solution other than doing this manually.

# Also make sure that NO jobs are running while you are
# doing this!

# Due to:

#Jul 11 23:05:49 zfs-archive1 genunix: [ID 881230 kern.warning] WARNING: sd4101:a minor 0x40140 too big for 32-bit applications
#Jul 11 23:05:49 zfs-archive1 unix: [ID 665567 kern.warning] WARNING: kstat_create('sd', 4104, 'fm'): namespace collision

# Steffen

#
# manage /etc/path_to_inst and remove
# all inactive scsi/iscsi device entries
#
while(<STDIN>)
{
	my $line = $_;
	
	#
	# "/scsi_vhci/disk@g6000402001fc12b06de5061700000000" 11 "sd"
	#
	if ( $line =~ /^\"(\/scsi_vhci\/disk\@.*?)\" (\d+) \"(\S+)\"/ )
	{
		#
		# check if the device is in /devices/scsi_vhci, if so
		# then it is active, keep it
		#
		my $scsi_vhci_dsk = $1;
		my $scsi_vhci_minor = $2;
		my $scsi_vhci_drv = $3;
		#print "$scsi_vhci_dsk minor: $scsi_vhci_minor driver: $scsi_vhci_drv\n";
		my $dev_dir = "/devices/$scsi_vhci_dsk";
		if ( -d $dev_dir )
		{
			print $line;
		}
	}
	else
	{
		print $line;
	}
}

