#!/usr/bin/perl

#
# generate next number (used for tcp server sockets)
# using file locking to guarantee a client to get a
# usable unique number between 12000 and 12500
#

use strict;
use Data::Dumper;
use Fcntl ':flock',':seek'; # Import LOCK_* constants

use FindBin;
use lib "$FindBin::Bin/lib";
use lib "$FindBin::Bin/etc";

use common;
use config;

$cfg{'verbose'} = 1;

my $port_min = 12000;
my $port_max = 12500;

my $lock_file = $cfg{'state_directory'}."/port.lock";
my $data_file = $cfg{'state_directory'}."/port.dat";

#
# open and lock lock file
#
open(my $fh_lock, '>>', $lock_file) or die "Could not open $lock_file: $!";
flock($fh_lock, LOCK_EX);

#
# critical section
#
if ( ! -e $data_file )
{
	open(my $fh_data, '>>', $data_file) or die "Could not open $data_file: $!";
	close($fh_data) or die "Could not close $data_file: $!";
}
open(my $fh_data, '+<', $data_file) or die "Could not open $data_file: $!";
my $port = <$fh_data>;
chomp($port);
if ( defined($port) )
{
	$port = $port + 1;
	if ( $port > $port_max )
	{
		$port = $port_min;
	}
}
else
{
	$port = $port_min;
}
truncate($fh_data,0);
seek($fh_data,0,SEEK_SET);
print $fh_data "$port\n";
close($fh_data) or die "Could not close $data_file: $!";

# output result via stdout
print "$port\n";

#
# unlock lock file
#
close($fh_lock) or die "Could not close $lock_file: $!";

exit(1);

