#
# tkzfs windows platform code
#
package tkzfswin;
use strict;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(
);

use Data::Dumper;
use English;
use Sys::Hostname;
use Win32::OLE('in');
use constant wbemFlagReturnImmediately => 0x10;
use constant wbemFlagForwardOnly => 0x20;

use FindBin;
use lib "$FindBin::Bin";
use config;
use tkzfsutil;
use tkzfsrepo;

my $win32_launched_diskmgmt = 0;

#-------------------------------------------------------------------------------
# microsoft iscsi software initiator
#-------------------------------------------------------------------------------
sub win32_lun_map_ms_iscsi
{
	my ($w_main) = @_;

	win32_lun_map($w_main, "ms_iscsi");
}

sub win32_lun_lookup_ms_iscsi
{
	my ($w_main) = @_;
	win32_lun_lookup($w_main, "ms_iscsi");
}

sub win32_lun_umap_ms_iscsi
{
	my ($w_main) = @_;
	win32_lun_umap($w_main, "ms_iscsi");
}

#-------------------------------------------------------------------------------
# qlogic iscsi software initiator
#-------------------------------------------------------------------------------
sub win32_lun_map_qla_iscsi
{
	my ($w_main) = @_;
	win32_lun_map($w_main, "qla_iscsi");
}

sub win32_lun_lookup_qla_iscsi
{
	my ($w_main) = @_;
	win32_lun_lookup($w_main, "qla_iscsi");
}

sub win32_lun_umap_qla_iscsi
{
	my ($w_main) = @_;
	win32_lun_umap($w_main, "qla_iscsi");
}



#-------------------------------------------------------------------------------
# microsoft windows lookup of a LUN
#-------------------------------------------------------------------------------
sub win32_lun_lookup
{
	my ($w_main, $hba) = @_;

	my $tree_item = $cfg{'tree_item'};
	
	my $properties = $cfg{'tree'}{$tree_item}{'properties'};
	my $repo_host = $$properties{'_tkzfs_host'};
	my $zpool = $$properties{'_tkzfs_zpool'};
	my $zvol = $$properties{'_tkzfs_zfs_name'};

	logger("lookup> using ms_iscsi for zvol: $zvol");

	#
	# get the iqn name of the target from the repository
	#
	my $iqn_name = zfs_repo_iscsi_target_get_iname($repo_host, $zpool, $zvol);
	if ( !defined($iqn_name) || $iqn_name !~ /^iqn./ )
	{
		logger("lookup_err> unable to determine iscsi qualified name (iqn) from iscsi target, expecting iqn. name");
		return;
	}
	logger("lookup> using iqn: $iqn_name");

	#
	# hba specific target lookup
	#
	my $is_mapped = 0;
	
	if ( $hba eq 'ms_iscsi' )
	{
		#
		# the target is just a record that is added to the MS initiator
		#
		logger("map_lookup> lookup target");
		$is_mapped = ms_iscsi_target_lookup($iqn_name);
	}
	elsif ( $hba eq 'qla_iscsi' )
	{
		$is_mapped = qla_iscsi_lookup_target($iqn_name);
	}
	else
	{
		logger("lookup_err> unknown hba specification");
		return;
	}
	logger("lookup> is_mapped: $is_mapped");
	
	return $is_mapped;
}

#-------------------------------------------------------------------------------
# microsoft windows map of a LUN
#-------------------------------------------------------------------------------
sub win32_lun_map
{
	my ($w_main, $hba) = @_;
	
	my $tree_item = $cfg{'tree_item'};
	
	my $properties = $cfg{'tree'}{$tree_item}{'properties'};
	my $repo_host = $$properties{'_tkzfs_host'};
	my $zpool = $$properties{'_tkzfs_zpool'};
	my $zvol = $$properties{'_tkzfs_zfs_name'};

	logger("map> using ms_iscsi for zvol: $zvol");

	#
	# connect to WMI
	#
	my $objWMIService = Win32::OLE->GetObject("winmgmts:\\\\.\\root\\CIMV2");
	if ( !defined($objWMIService) )
	{
		logger("disk_err> WMI connection failed to localhost");
		return undef;
	}

	#
	# run diskmgmt api gui
	#
	win32_diskmgmt();

	#
	# get the iqn name of the target from the repository
	#
	my $iqn_name = zfs_repo_iscsi_target_get_iname($repo_host, $zpool, $zvol);
	if ( !defined($iqn_name) || $iqn_name !~ /^iqn./ )
	{
		logger("map_err> unable to determine iscsi qualified name (iqn) from iscsi target, expecting iqn. name");
		return;
	}
	
	#
	# hba specific logon/bind to target
	#
	if ( $hba eq 'ms_iscsi' )
	{
		#
		# the target is just a record that is added to the MS initiator
		#
		logger("map> lookup target");
		if ( !ms_iscsi_target_lookup($iqn_name) )
		{
			logger("map> target not found, adding it");
			ms_iscsi_target_add($iqn_name, $cfg{'repositories'}{$repo_host}{'iscsi_target_ip'});
		}
		
		#
		# establishing a session is what binds the initiator with the target
		#
		my $isessions = ms_iscsi_sessions();
		if ( exists($$isessions{$iqn_name}) )
		{
			logger("map> the target has already a connected session");
		}
		else
		{
			logger("map> login to target...");
			ms_iscsi_target_login($iqn_name);
			
			#
			# make sure we have a session before proceeding
			#
			$isessions = ms_iscsi_sessions();
			if ( !exists($$isessions{$iqn_name}) )
			{
				logger("map_err> establishing a connection with the target failed");
				return;
			}
			logger("map> login to target successful");
		}
	}
	elsif ( $hba eq 'qla_iscsi' )
	{
		#
		# lookup the target
		#
		logger("map> lookup target");
		if ( !qla_iscsi_lookup_target($iqn_name) )
		{
			#
			# create persistent target
			#
			logger("map> target not found, adding it");
			qla_iscsi_target_add($iqn_name, $cfg{'repositories'}{$repo_host}{'iscsi_target_ip'});
		}
	}
	else
	{
		logger("map_err> unknown hba specification");
		return;
	}

	#
	# rescan local LUNs using diskpart
	#
	logger("map> disk scan begin");
	win32_diskpart_rescan();
	sleep(5);
	logger("map> disk scan complete");
	
	#
	# lookup GUID of LUN in the repository, one LUN per target
	#
	logger("map> lookup GUID of first LUN of selected target");
	my $guid = zfs_repo_iscsi_target_lun0_guid($repo_host, $zpool, $zvol);
	if ( !defined($guid) )
	{
		logger("map_err> unable to determine GUID from remote LUN");
		return;
	}
	logger("map> LUN GUID: $guid");

	#
	# map the remote GUID to a local physical disk (\\.\PHYSICALDRIVE#)
	#
	my $physical_disk = win32_diskguid_lookup($objWMIService, $guid);
	if ( !defined($physical_disk) )
	{
		logger("map_err> unable to find GUID ($guid) while enumerating physical disks");
		return;
	}
	logger("map> Physical disk: $physical_disk");
	
	#
	# derive the actual disk number from the last digits of the PHYSICALDRIVE
	#
	my $disk_no;
	if ( $physical_disk =~ /.*(\d+)$/ )
	{
		$disk_no = $1;
	}
	if ( !defined($disk_no) )
	{
		logger("map_err> unable to determine disk number from physical disk (last digit??)");
		return;
	}
	logger("map> found disk #$disk_no");

	#
	# get volume number and drive letter (optionally map the next available drive letter, this
	# all assumes that there is only a single primary partition on a disk - this is windows!)
	#
	my ($volume_no, $drive_letter);
	my $disk_retry = 0;
	my $disk_retry_max = 10;
	while ( $disk_retry < $disk_retry_max )
	{
		logger("map> getting disk and volume information, expect a delay... (attempt: $disk_retry of $disk_retry_max)");
		($volume_no, $drive_letter) = win32_diskpart_select_disk_get_info($disk_no);
		if ( defined($volume_no) )
		{
			last;
		}
		$disk_retry++;
		logger("map> sleeping 15 seconds before retrying $disk_retry of $disk_retry_max . . .");
		sleep(15);
	}
	if ( defined($volume_no) )
	{
		logger("map> found volume #$volume_no with driver letter: $drive_letter");
	}
	else
	{
		logger("map_err> unable to determine first volume number from disk number, disk is not ready in this host");
		return;
	}
	
	if ( !defined($drive_letter) )
	{
		$drive_letter = win32_drive_letter_free($objWMIService);
		logger("map> found next free drive letter: $drive_letter");
		
		if ( defined($drive_letter) )
		{
			logger("map> assigning drive letter: $drive_letter to volume_no: $volume_no");
			win32_diskpart_assign_drive_letter($volume_no, $drive_letter);

			logger("map> sleeping for 15 seconds, drive letter assignment . . .");
			sleep(15);
		}
		else
		{
			logger("map_err> unable to determine next available drive letter, possibly all used");
			return;
		}
	}

	#
	# at this point, just to be sure double-check the drive letter
	#
	logger("map> validating drive letter assignment (looking for drive letter $drive_letter on disk: $disk_no)");
	my ($volume_no_check, $drive_letter_check) = win32_diskpart_select_disk_get_info($disk_no);
	if ( $drive_letter ne $drive_letter_check )
	{
		logger("map_err> expected drive letter: $drive_letter but got instead $drive_letter_check");
		return;
	}
	if ( $volume_no ne $volume_no_check )
	{
		logger("map_err> expected volume_no: $volume_no but got instead $volume_no_check");
		return;
	}
	logger("map> drive letter mapping is ok");
	
	#
	# VSS/SAN based snapshots are 'media write protected', must use writeprot to disable that, and diskpart
	# to clear out several other volume attributes. The mapped drive letter is used by writeprot, the change
	# applies to the disk where the volume resides (if mbr is used to allocate disk) otherwise the change applies
	# to the volume itself.
	#
	win32_writeprot_off($volume_no, $drive_letter);
	
	#
	# open explorer and lead us to the files...
	#
	my $cmd = "explorer /e,/root,/select,$drive_letter:";
	cmd_exec($cmd);
}

#-------------------------------------------------------------------------------
# microsoft windows umap of a LUN
#-------------------------------------------------------------------------------
sub win32_lun_umap
{
	my ($w_main, $hba) = @_;
	
	my $tree_item = $cfg{'tree_item'};
	
	my $properties = $cfg{'tree'}{$tree_item}{'properties'};
	my $repo_host = $$properties{'_tkzfs_host'};
	my $zpool = $$properties{'_tkzfs_zpool'};
	my $zvol = $$properties{'_tkzfs_zfs_name'};

	logger("umap> using ms_iscsi for zvol: $zvol");

	#
	# get the iqn name of the target from the repository
	#
	my $iqn_name = zfs_repo_iscsi_target_get_iname($repo_host, $zpool, $zvol);
	if ( !defined($iqn_name) || $iqn_name !~ /^iqn./ )
	{
		logger("umap_lookup_err> unable to determine iscsi qualified name (iqn) from iscsi target, expecting iqn. name");
		return;
	}

	#
	# hba specific target logout
	#
	if ( $hba eq 'ms_iscsi' )
	{
		#
		# to logout, we need the session id for the target
		#
		my $isessions = ms_iscsi_sessions();
		if ( exists($$isessions{$iqn_name}) )
		{
			my $session_id = $$isessions{$iqn_name};
			logger("umap> the target has a connected session with session_id: $session_id");
			
			#
			# logout from the session
			#
			ms_iscsi_target_logout($session_id);
		}
		else
		{
			logger("umap> the target has no connected session");
		}
		
		#
		# remove the target from the initiator
		#
		ms_iscsi_target_remove($iqn_name);
	}
	elsif ( $hba eq 'qla_iscsi' )
	{
		#
		# lookup the target
		#
		logger("map> lookup target");
		if ( qla_iscsi_lookup_target($iqn_name) )
		{
			#
			# remove persistent target
			#
			logger("map> target found, removing it");
			
			my $hba_targets = qla_iscsi_persistent_targets();
			my $hba_target = $$hba_targets{$iqn_name};

			logger("umap> hba target $hba_target");
			
			qla_iscsi_target_remove($hba_target);
		}
		else
		{
			logger("umap> the target was not connected");
		}
	}
	else
	{
		logger("umap_lookup_err> unknown hba specification");
		return;
	}
}



#-------------------------------------------------------------------------------
# ms iscsi: target lookup
#-------------------------------------------------------------------------------
sub ms_iscsi_target_lookup
{
	my ($iqn_name) = @_;
	
	my $cmd = "iscsicli ListTargets";
	my $out = cmd_exec($cmd);
	my @lines = split(/\n/,$out);
	foreach my $line ( @lines )
	{
		if ( $line =~ /(iqn.*)/ )
		{
			my $iqn = $1;
			if ( $iqn eq $iqn_name )
			{
				return 1;
			}
		}
	}
	return 0;
}

#-------------------------------------------------------------------------------
# ms iscsi: target add
#-------------------------------------------------------------------------------
sub ms_iscsi_target_add
{
	my ($iqn_name, $portal_ip) = @_;
	
	my $cmd = "iscsicli QAddTarget $iqn_name $portal_ip";
	my $out = cmd_exec($cmd);
}

#-------------------------------------------------------------------------------
# ms iscsi: target remove
#-------------------------------------------------------------------------------
sub ms_iscsi_target_remove
{
	my ($iqn_name) = @_;
	
	my $cmd = "iscsicli RemoveTarget $iqn_name";
	my $out = cmd_exec($cmd);
}

#-------------------------------------------------------------------------------
# ms iscsi: target login (this creates a tcp session, i.e. binding takes place)
#-------------------------------------------------------------------------------
sub ms_iscsi_target_login
{
	my ($iqn_name) = @_;

	my $cmd = "iscsicli QLoginTarget $iqn_name";
	my $out = cmd_exec($cmd);
}

#-------------------------------------------------------------------------------
# ms iscsi: get a mapping of target and session identifiers (ms only supports
# a single session per target, otherwise the hash would have an array as its value)
#
# Session Id             : ffffffff8873774c-4000013700000006
# Target Name            : iqn.1986-03.com.sun:02:b0493017-d05d-e8a1-df3f-a0c50c60262f
#-------------------------------------------------------------------------------
sub ms_iscsi_sessions
{
	my %isessions;
	
	my $cmd = "iscsicli SessionList";
	my $out = cmd_exec($cmd);
	my @lines = split(/\n/,$out);
	
	my $session_id;
	my $target_name;
	foreach my $line ( @lines )
	{
		if ( $line =~ /^Session Id\s*:\s(.*)/i )
		{
			$session_id = $1;
		}
		elsif ( $line =~ /^Target Name\s*:\s(.*)/i )
		{
			$target_name = $1;
			
			$isessions{$target_name} = $session_id;
		}
	}
	return \%isessions;
}

#-------------------------------------------------------------------------------
# ms iscsi: target logout (disconnects TCP session)
#-------------------------------------------------------------------------------
sub ms_iscsi_target_logout
{
	my ($session_id) = @_;

	my $cmd = "iscsicli LogoutTarget $session_id";
	my $out = cmd_exec($cmd);
}


#-------------------------------------------------------------------------------
# qlogic iscsi target lookup
#-------------------------------------------------------------------------------
sub qla_iscsi_lookup_target
{
	my($iname) = @_;
	
	my $hba_targets = qla_iscsi_persistent_targets();
	if ( exists($$hba_targets{$iname}) )
	{
		logger("hba> lookup of $iname successful");
		return 1;
	}

	logger("hba> lookup of $iname not found");
	return 0;
}

#-------------------------------------------------------------------------------
# qlogic iscsi targets
#-------------------------------------------------------------------------------
sub qla_iscsi_persistent_targets
{
	my $this_host = hostname;

	my $cmd = $cfg{'os'}{$OSNAME}{'qla_cmd'}.' -ps '.$cfg{'host'}{$this_host}{'iscsi_hba_instance'};
	my $data = cmd_exec($cmd);
	my %targets;
	my $id;
	foreach my $line (split(/\n/, $data))
	{
		if ( $line =~ /Target ID: (\d+)/ )
		{
			$id = $1;
		}
		if ( $line =~ /ISCSI Name: (.*)/ )
		{
			my $iname = $1;
			if ( $iname eq "" )
			{
				next;
			}
			$targets{$1} = $id;
		}
	}
	return \%targets;
}

#-------------------------------------------------------------------------------
# qlogic iscsi target add
#-------------------------------------------------------------------------------
sub qla_iscsi_target_add
{
	my ($iqn_name, $portal_ip) = @_;
	
	my $this_host = hostname;

	my $cmd = $cfg{'os'}{$OSNAME}{'qla_cmd'}.
		' -pa '.$cfg{'host'}{$this_host}{'iscsi_hba_instance'}.
		" $portal_ip -INAME \"$iqn_name\"";
	cmd_exec($cmd);
}

#-------------------------------------------------------------------------------
# qlogic iscsi target remove
#-------------------------------------------------------------------------------
sub qla_iscsi_target_remove
{
	my ($hba_name) = @_;
	
	my $this_host = hostname;

	my $cmd = $cfg{'os'}{$OSNAME}{'qla_cmd'}.
		' -pu '.$cfg{'host'}{$this_host}{'iscsi_hba_instance'}.
		" \"$hba_name\"";
	cmd_exec($cmd);
}


#-------------------------------------------------------------------------------
# invoke diskmgmt.msc to maintain the 'list volume' 
# index order consistent when invoking diskpart multiple times
# http://support.microsoft.com/kb/937252
#
# I have not been able to determine the disk/partition/volume association
# in wmi and the volume is what exposes the 'assign' drive letter method.
# only launch this once per session (the user better not close it!)
#-------------------------------------------------------------------------------
sub win32_diskmgmt
{
	if ( !$win32_launched_diskmgmt )
	{
		my $cmd = "start /min diskmgmt.msc";
		cmd_exec($cmd);
		logger("map> diskmgmt launched (do NOT close), sleeping for 5 seconds to let things settle...");
		sleep(5);
		
		$win32_launched_diskmgmt = 1;
	}
}

#-------------------------------------------------------------------------------
# diskpart: rescan
#-------------------------------------------------------------------------------
sub win32_diskpart_rescan
{
	logger("diskpart> rescanning for new LUNs, this can take time...");

	my $cmd = "diskpart /s $cfg{'root_dir'}/win32/diskpart_rescan.txt";
	cmd_exec($cmd);

	logger("diskpart> rescanning for new LUNs, complete");
}

#-------------------------------------------------------------------------------
# given a disk GUID identifier, enumerate via WMI the win32_diskdrive objects
# and interrogate them using disk-vpd to see if there is a matching guid. The
# physical drive number will be later used with diskpart...
#-------------------------------------------------------------------------------
sub win32_diskguid_lookup
{
	my ($objWMIService, $guid) = @_;
	
	logger("disk> enum via Win32_DiskDrive the disk devices and look for GUID $guid");

	my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_DiskDrive", "WQL",
		  wbemFlagReturnImmediately | wbemFlagForwardOnly);
	
	my $physical_disk;
	foreach my $objItem (in $colItems)
	{
		logger("disk> $objItem->{Name}");
		
		my $cmd = "$cfg{'root_dir'}/win32/disk-vpd $objItem->{Name}";
		my $data = cmd_exec($cmd);
		foreach my $line (split(/\n/, $data))
		{
			if ( $line =~ /GUID: $guid/ )
			{
				$physical_disk = $objItem->{Name};
				last;
			}
		}
	}
	return $physical_disk;
}

#-------------------------------------------------------------------------------
# diskpart: select disk and gets its info, such as volume number and drive letter
#
#  Volume ###  Ltr  Label        Fs     Type        Size     Status     Info
#  ----------  ---  -----------  -----  ----------  -------  ---------  --------
#  Volume 0     E   fs001-d-equ  NTFS   Partition    850 GB  Healthy            
#-------------------------------------------------------------------------------
sub win32_diskpart_select_disk_get_info
{
	my ($disk_no) = @_;
	
	#
	# get disk details via diskpart
	#
	my $fname_diskdetail = "$cfg{'root_dir'}/diskpart_diskdetail_$$.txt";
	if ( !open(FH,">$fname_diskdetail") )
	{
		logger("map_err> unable to write to $fname_diskdetail: $!");
		return;
	}
	print FH "list disk\n";
	print FH "select disk $disk_no\n";
	print FH "detail disk\n";
	close FH;

	my $cmd = "diskpart /s $fname_diskdetail";
	my $disk_detail = cmd_exec($cmd);
	my $volume_no;
	my $drive_letter;
	foreach my $line (split(/\n/, $disk_detail))
	{
		if ( $line =~ /Volume (\d+)\s{5}([A-Z]?)/ )
		{
			$volume_no = $1;	# required
			$drive_letter = $2;	# optional
			if ( $drive_letter eq '' )
			{
				$drive_letter = undef;
			}
			last;
		}
	}
	unlink($fname_diskdetail);
	return ($volume_no, $drive_letter);
}

#-------------------------------------------------------------------------------
# get the next available drive letter, skipping floppy disks...
#-------------------------------------------------------------------------------
sub win32_drive_letter_free
{
	my ($objWMIService) = @_;
	
	#
	# ok, this is nuts, let's do the alphabet c,d,e,f,g... (skip A: and B:)
	#
	my %alphabet;
	for(my $i=3; $i<=26; $i++)
	{
		my $letter = sprintf("%c", $i+64);
		$alphabet{$letter} = undef;
	}

	#
	# now enum via win32_volume (wmi) the current drive letters
	# and remove those from the available alphabet soup defined above
	#
	logger("drive_letter> enum via Win32_Volume, looking for a free drive letter");

	my $colItems = $objWMIService->ExecQuery("SELECT * FROM Win32_Volume", "WQL",
		  wbemFlagReturnImmediately | wbemFlagForwardOnly);

	foreach my $objItem (in $colItems)
	{
		logger("drive_letter> $objItem->{Name}: $objItem->{DriveLetter} in use");
		my $letter = $objItem->{DriveLetter};
		chop($letter);
		if ( exists($alphabet{$letter}) )
		{
			delete $alphabet{$letter};
		}
	}
	logger("map> drive_letter drive letters: ".join(',',sort keys %alphabet));
	my @letters = sort keys %alphabet;
	my $drive_letter = pop(@letters);
	
	return $drive_letter;
}

#-------------------------------------------------------------------------------
# diskpart: select volume_no and assign drive letter
#-------------------------------------------------------------------------------
sub win32_diskpart_assign_drive_letter
{
	my ($volume_no, $drive_letter) = @_;

	#
	# invoke diskpart and assign the new drive letter
	#
	my $fname_assign = "$cfg{'root_dir'}/diskpart_assign_$$.txt";
	if ( !open(FH,">$fname_assign") )
	{
		logger("diskpart_err> unable to write to $fname_assign: $!");
		return;
	}
	print FH "list volume\n";
	print FH "select volume $volume_no\n";
	print FH "assign letter=$drive_letter\n";
	close FH;

	my $cmd = "diskpart /s $fname_assign";
	cmd_exec($cmd);
	
	unlink($fname_assign);
}

#-------------------------------------------------------------------------------
# writeprot clears the volume's readonly bit (might need /unsafe if there are
# multiple volumes on a disk (normally we don't do that)).
#-------------------------------------------------------------------------------
sub win32_writeprot_off
{
	my ($volume_no, $drive_letter) = @_;
	
	#
	# make volume read/writeable
	#
	my $cmd = "$cfg{'root_dir'}/win32/writeprot /vol $drive_letter: /rw";
	cmd_exec($cmd);
	
	#
	# for documentation purposes, display the volume attributes
	#
	#
	# invoke diskpart and assign the new drive letter
	#
	my $fname_attr = "$cfg{'root_dir'}/diskpart_attr_$$.txt";
	if ( !open(FH,">$fname_attr") )
	{
		logger("diskpart_err> unable to write to $fname_attr: $!");
		return;
	}
	print FH "list volume\n";
	print FH "select volume $volume_no\n";
	print FH "detail volume\n";
	close FH;

	my $cmd = "diskpart /s $fname_attr";
	cmd_exec($cmd);
	
	unlink($fname_attr);
}

return 1;
