#!/bin/sh

#
# since fetching all of these properties can take time, we provide the following
# interfaces:
#	refresh: force data refresh including cache
#	fast: get data from cache, if there is a cache
#

ROOT=$(dirname $0)
. $ROOT/zfs_tree.conf

cmd="$1"

function query
{
	#
	# list public properties
	#
	zfs get -r -H all $zpools

	#
	# list unpublished properties (at least what we found)
	#
	#zfs get -r -H name,guid,iscsioptions,stmf_sbd_lu $zpools
	zfs get -r -H name,guid,iscsioptions $zpools
}

function refresh
{
		mach_name=$(hostname)
		query | awk '{print "'$mach_name'/" $0}' > /tmp/$$.cache
		mv /tmp/$$.cache $ROOT/zfs_tree.cache
}

function output
{
		cat $ROOT/zfs_tree.cache
}

case "$cmd" in
	fast)
		if [ ! -e $ROOT/zfs_tree.cache ]; then
			refresh
		fi
		output
		;;
	refresh)
		refresh
		output
		;;
	*)
		echo "$0 fast|refresh"
		exit 1
esac

